"""
    Downloader class.
"""
import os
import sys


class Downloader:
    output_directory = None
    connector = None
    subprocess = None
    download_queue = None
    download_count = None
    max_per_list = 10

    def __init__(self, connector, output_directory, subprocess=None):
        """
            Initiliaze Downloader

            Parameters:
                connector: connector to Grooveshark api
                subprocess: spawn new processes
                output_directory: where files will be downloaded
        """
        if subprocess is not None:
            self.subprocess = subprocess
        else:
            import subprocess
            self.subprocess = subprocess

        self.output_directory = output_directory
        self.connector = connector
        self.download_queue = []
        self.download_count = 0

    def download_playlist(self, playlist_id=None):
        """
            Download Playlist

            Parameters:
                playlist_id: the playlist id
        """
        songs = []
        if (playlist_id is not None):
            plist = self.connector.get_playlist_from_id(playlist_id)
            songs = plist["Songs"] if ("Songs" in plist) else []
        else:
            for playlist in self.download_queue:
                plist = self.connector.get_playlist_from_id(
                    playlist['PlaylistID']
                )
                songs = songs + plist["Songs"]

        self.download_queue = songs
        self.download()

    def download_song(self, filename, song):
        """
            Download song

            Parameters:
                filename: the filename
                song: dictionary with song informations
        """
        print("Downloading: %s" % (filename))
        stream_key = self.connector.get_stream_key_from_song_id(song["SongID"])
        if stream_key == []:
            print("Failed to retrieve stream key!")
            return False

        #Run wget to download the song
        wget = "wget --progress=dot --post-data=streamKey=%s" \
            " -O \"%s\" \"http://%s/stream.php\"" % (
                stream_key["streamKey"], filename, stream_key["ip"]
            )
        cmd = wget + " 2>&1 | grep --line-buffered \"%\" |" \
            "sed -u -e \"s,\.,,g\" | awk '{printf(\"\b\b\b\b%4s\", $2)}'"
        process = self.subprocess.Popen(cmd, shell=True)

        try:
            process.wait()
            print("\nDownloaded")
            self.download_count += 1
        except BaseException:
            print("Download cancelled. File deleted.")
            os.remove(filename)
            return False

        return True

    def prepare(self, query, type=None):
        """
            Prepare downloads

            Parameters:
                query: the search query
                type: the search type
                    (types are 'Songs', 'Artists', 'Albums' or 'Playlists')
        """
        result = self.connector.search(query, type)
        self.__display_raw_input__(
            result,
            type,
            "Press \"n\" for next page, "
            "\"Number id\" to add element in queue, "
            "\"q\" for quit and download: "
        )

    def __display_raw_input__(self, result, type, input_text):
        """
            Display prompt to choose songs, playlist, etc...

            Parameters:
                result: the result from the connector
                type: the search type
                    (types are 'Songs', 'Artists', 'Albums' or 'Playlists')
                input_text: texte for the raw input
        """
        try:
            for idx, data in enumerate(result):
                key = None
                if (type != "Playlists"):
                    print(
                        "%d - Album: %sSong: %s - %s" % (
                            idx,
                            data["AlbumName"].ljust(40),
                            data["ArtistName"],
                            data["SongName"]
                            if "SongName" in data else data["Name"]
                        )
                    )
                else:
                    print(
                        "%d - Playlist: %sAuthor: %s with %s songs" % (
                            idx,
                            data["Name"].ljust(40),
                            data["FName"],
                            data["NumSongs"]
                        )
                    )

                is_last = (idx == (len(result) - 1))
                if ((idx != 0 and idx % self.max_per_list == 0) or is_last):
                    while (key is not False):
                        key = input(input_text)
                        if (key == "q"):
                            raise StopIteration
                        elif (key == "n" and is_last is False):
                            key = False
                        elif (key.isdigit()):
                            key = int(key)
                            if (key >= 0 and key <= len(result)):
                                added_data = result[key]
                                self.download_queue.append(added_data)
                                print(
                                    "%s added" %
                                    added_data["SongName"]
                                    if "SongName" in added_data
                                    else added_data["Name"]
                                )
                                continue
        except StopIteration:
            pass
        except Exception as inst:
            print("Unexpected error:", sys.exc_info()[0])
            raise inst

    def download(self):
        """
            Download files
        """
        if (self.download_queue != []):
            for file in self.download_queue:
                filename = (
                    "%s/%s - %s.mp3" %
                    (
                        self.output_directory,
                        file["ArtistName"],
                        file["SongName"]
                        if "SongName" in file else file["Name"]
                    )
                )

                if (os.path.exists(filename) is not True):
                    self.download_song(filename, file)
                else:
                    self.download_count += 1
            return True
        return False

    def has_downloaded(self):
        return False if self.download_count == 0 else True



#!/usr/bin/env python

import os
import sys

from setuptools import setup
from distutils.core import Command
from subprocess import call


class NoseCommand(Command):
    description = "run test suite"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        status = self._run_tests()
        sys.exit(status)

    def _run_tests(self):
        try:
            import nose
            nose
        except ImportError:
            print('Missing "nose" library. You can install it using pip: '
                  "pip install nose")
            sys.exit(1)

        retcode = call("nosetests")
        sys.exit(retcode)
        return not res.wasSuccessful()


class Pep8Command(Command):
    description = "run pep8 script"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            import pep8
            pep8
        except ImportError:
            print('Missing "pep8" library. You can install it using pip: '
                  "pip install pep8")
            sys.exit(1)

        cwd = os.getcwd()
        retcode = call(("pep8 %s/groove/ %s/tests/ %s/setup.py" %
                        (cwd, cwd, cwd)).split(" "))
        sys.exit(retcode)


setup(
    name="Groove-dl",
    version="0.1",
    description="Grooveshark song downloader",
    author="Pierre Rambaud (GoT)",
    url="https://github.com/PierreRambaud/groove-dl",
    license="GPLv3",
    scripts=["groove-dl"],
    packages=["groove"],
    install_requires=[],
    tests_require=[
            "mock",
            "nose",
            "pep8"
        ],
    cmdclass={
        "pep8": Pep8Command,
        "nose": NoseCommand
        },
    )



"""
    Connector class to communicate with
    grooveshark API.
"""
import random
import string
import uuid
import hashlib
import json
import sys
import http.client


class Connector:
    token = None
    client = None
    js_queue = None
    url = "grooveshark.com"
    request_url = "/more.php"
    encoding = "utf-8"
    headers = {
        "country": {
            "CC1": 72057594037927940,
            "CC2": 0,
            "CC3": 0,
            "CC4": 0,
            "ID": 57,
            "IPR": 0,
        },
        "privacy": 0,
    }
    html_client = (
        "htmlshark",
        "20130520",
        "nuggetsOfBaller",
        {
            "User-Agent": ("Mozilla/5.0 (X11; Linux x86_64; rv:17) "
                           "Gecko/20130917 Firefox/17 Iceweasel/17.0.29"),
            "Content-Type": "applicaiton/json"
        }
    )

    def __init__(self, client=None):
        """
            Initiliaze Connector

            Parameters:
                client: the http client
        """
        if client is not None:
            self.client = client
        else:
            self.client = http.client.HTTPSConnection(self.url)

        self.headers["session"] = ("".join(
            random.choice(
                string.digits + string.ascii_letters[:6]
            ) for x in range(32)
        )).lower()

        self.headers["uuid"] = str.upper(str(uuid.uuid4()))

        self.js_queue = [
            "jsqueue",
            "20130520",
            "chickenFingers"
        ]
        self.js_queue.append(
            {
                "User-Agent": self.html_client[3]["User-Agent"],
                "Referer": "http://%s/JSQueue.swf?%s" % (
                    self.url, self.js_queue[1]
                ),
                "Content-Type": "application/json"
            }
        )

    def get_token(self):
        """
            This method retrieves the local token for the session
            which is used to generate tokens for each request.
        """
        if (self.token is None):
            options = self.__get_request_options__()
            options["method"] = "getCommunicationToken"
            try:
                response = self.__execute_query__(
                    self.request_url,
                    options,
                    self.html_client[3]
                )
                self.token = response["result"]
            except Exception as inst:
                print("Unexpected error:", sys.exc_info()[0])
                raise inst

        return self.token

    def get_playlist_from_id(self, playlist_id):
        """
            Get playlist songs from playlist id

            Parameters:
                playlist_id: the playlist id
        """
        options = self.__get_request_options__()
        options["method"] = "getPlaylistByID"
        options["parameters"]["playlistID"] = playlist_id
        options["header"]["token"] = self.__prepare_token__(
            options["method"],
            self.html_client[2]
        )

        try:
            response = self.__execute_query__(
                self.request_url,
                options,
                self.html_client[3]
            )
            return response["result"]
        except Exception as inst:
            print("Unexpected error:", sys.exc_info()[0])
            raise inst

    def search(self, query, type="Songs"):
        """
            This method searches for a song using a provided search term

            Parameters:
                query: the search query
                type: the search type
                    (types are 'Songs', 'Artists', 'Albums' or 'Playlists')
        """
        haystack = ["Songs", "Artists", "Albums", "Playlists"]
        if (type not in haystack):
            type = "Songs"

        options = self.__get_request_options__()
        options["method"] = "getResultsFromSearch"
        options["parameters"]["query"] = query
        options["parameters"]["type"] = type
        options["header"]["token"] = self.__prepare_token__(
            options["method"],
            self.html_client[2]
        )

        try:
            response = self.__execute_query__(
                self.request_url,
                options,
                self.html_client[3]
            )
            return response["result"]["result"]
        except Exception as inst:
            print("Unexpected error:", sys.exc_info()[0])
            raise inst

    def get_stream_key_from_song_id(self, song_id):
        """
            Get the streamKey needed to request the download link to the MP3

            Parameters:
                song_id: the songID of the song you want to download.
        """
        options = self.__get_request_options__()
        options["parameters"]["type"] = 8
        options["parameters"]["mobile"] = False
        options["parameters"]["prefetch"] = False
        options["parameters"]["songID"] = song_id
        options["parameters"]["country"] = ",".join(self.headers["country"])
        options["header"]["client"] = self.js_queue[0]
        options["header"]["clientRevision"] = self.js_queue[1]
        options["method"] = "getStreamKeyFromSongIDEx"
        options["header"]["token"] = self.__prepare_token__(
            options["method"],
            self.js_queue[2]
        )

        try:
            response = self.__execute_query__(
                self.request_url,
                options,
                self.js_queue[3]
            )
            return response["result"]
        except Exception as inst:
            print("Unexpected error:", sys.exc_info()[0])
            raise inst

    def __get_request_options__(self):
        """
            Return request default options
        """
        options = {}
        options["parameters"] = {}
        options["parameters"]["secretKey"] = hashlib.md5(
            self.headers["session"].encode(self.encoding)
        ).hexdigest()
        options["header"] = self.headers.copy()
        options["header"]["client"] = self.html_client[0]
        options["header"]["clientRevision"] = self.html_client[1]
        options["header"]["Accept"] = "text/html"

        return options

    def __execute_query__(self, url, parameters, userAgent):
        """
            Execute query
            Parameters:
                url: Request url
                parameters: All requests parameters (headers, raw data, etc...)
                userAgent: the user agent
        """
        self.client.request(
            "POST",
            url,
            json.JSONEncoder().encode(parameters),
            userAgent
        )
        response = self.client.getresponse().read()
        return json.JSONDecoder().decode(response.decode(self.encoding))

    def __prepare_token__(self, method, secret):
        """
            Prepare token
            Parameters:
                method: Method name.
                secret: the secret key
        """
        rnd = ("".join(
            random.choice(string.hexdigits) for x in range(6)
        )).lower()
        s = "%s:%s:%s:%s" % (method, self.get_token(), secret, rnd)
        return rnd + hashlib.sha1(s.encode("utf-8")).hexdigest()
